/****************************************************************************
  LPC1343.h
****************************************************************************/

#ifndef LPC1343INCLUDED
#define LPC1343INCLUDED

#ifdef __cplusplus
 extern "C" {
#endif

#ifndef dword
#define dword unsigned long
#endif

/* SysTick fr alle LPC13xx,  10 ms System-Tick Downcounter, 24 Bit */
#define STCTRL  (*((volatile dword *) 0xE000E010 ))
#define STLOAD  (*((volatile dword *) 0xE000E014 ))
#define STVAL   (*((volatile dword *) 0xE000E018 ))
#define STCALIB (*((volatile dword *) 0xE000E01C ))

/* NVIC Register */
#define NVIC_ISER0   (*((volatile dword *) 0xE000E100 ))
#define NVIC_ISER1   (*((volatile dword *) 0xE000E104 ))

#define NVIC_ICER0   (*((volatile dword *) 0xE000E180 ))
#define NVIC_ICER1   (*((volatile dword *) 0xE000E184 ))

#define NVIC_ISPR0   (*((volatile dword *) 0xE000E200 ))
#define NVIC_ISPR1   (*((volatile dword *) 0xE000E204 ))

#define NVIC_ICPR0   (*((volatile dword *) 0xE000E280 ))
#define NVIC_ICPR1   (*((volatile dword *) 0xE000E284 ))

#define NVIC_IABR0   (*((volatile dword *) 0xE000E300 ))
#define NVIC_IABR1   (*((volatile dword *) 0xE000E304 ))

#define NVIC_IPR0    (*((volatile dword *) 0xE000E400 ))
#define NVIC_IPR1    (*((volatile dword *) 0xE000E404 ))
#define NVIC_IPR2    (*((volatile dword *) 0xE000E408 ))
#define NVIC_IPR3    (*((volatile dword *) 0xE000E40C ))
#define NVIC_IPR4    (*((volatile dword *) 0xE000E410 ))
#define NVIC_IPR5    (*((volatile dword *) 0xE000E414 ))
#define NVIC_IPR6    (*((volatile dword *) 0xE000E418 ))
#define NVIC_IPR7    (*((volatile dword *) 0xE000E41C ))
#define NVIC_IPR8    (*((volatile dword *) 0xE000E420 ))
#define NVIC_IPR9    (*((volatile dword *) 0xE000E424 ))
#define NVIC_IPR10   (*((volatile dword *) 0xE000E428 ))
#define NVIC_IPR11   (*((volatile dword *) 0xE000E42C ))
#define NVIC_IPR12   (*((volatile dword *) 0xE000E430 ))
#define NVIC_IPR13   (*((volatile dword *) 0xE000E434 ))
#define NVIC_IPR14   (*((volatile dword *) 0xE000E438 ))

#define NVIC_STIR    (*((volatile dword *) 0xE000EF00 ))


/*
 * ==========================================================================
 * ---------- Interrupt Number Definition -----------------------------------
 * ==========================================================================
 */
typedef enum IRQn
{
/******  Cortex-M3 Processor Exceptions Numbers ***************************************************/
  NonMaskableInt_IRQn           = -14,      /*!< 2 Non Maskable Interrupt                         */
  MemoryManagement_IRQn         = -12,      /*!< 4 Cortex-M3 Memory Management Interrupt          */
  BusFault_IRQn                 = -11,      /*!< 5 Cortex-M3 Bus Fault Interrupt                  */
  UsageFault_IRQn               = -10,      /*!< 6 Cortex-M3 Usage Fault Interrupt                */
  SVCall_IRQn                   = -5,       /*!< 11 Cortex-M3 SV Call Interrupt                   */
  DebugMonitor_IRQn             = -4,       /*!< 12 Cortex-M3 Debug Monitor Interrupt             */
  PendSV_IRQn                   = -2,       /*!< 14 Cortex-M3 Pend SV Interrupt                   */
  SysTick_IRQn                  = -1,       /*!< 15 Cortex-M3 System Tick Interrupt               */

/******  LPC13xx Specific Interrupt Numbers *******************************************************/
  WAKEUP0_IRQn                  = 0,        /*!< All I/O pins can be used as wakeup source.       */
  WAKEUP1_IRQn                  = 1,        /*!< There are 40 pins in total for LPC17xx           */
  WAKEUP2_IRQn                  = 2,
  WAKEUP3_IRQn                  = 3,
  WAKEUP4_IRQn                  = 4,
  WAKEUP5_IRQn                  = 5,
  WAKEUP6_IRQn                  = 6,
  WAKEUP7_IRQn                  = 7,
  WAKEUP8_IRQn                  = 8,
  WAKEUP9_IRQn                  = 9,
  WAKEUP10_IRQn                 = 10,
  WAKEUP11_IRQn                 = 11,
  WAKEUP12_IRQn                 = 12,
  WAKEUP13_IRQn                 = 13,
  WAKEUP14_IRQn                 = 14,
  WAKEUP15_IRQn                 = 15,
  WAKEUP16_IRQn                 = 16,
  WAKEUP17_IRQn                 = 17,
  WAKEUP18_IRQn                 = 18,
  WAKEUP19_IRQn                 = 19,
  WAKEUP20_IRQn                 = 20,
  WAKEUP21_IRQn                 = 21,
  WAKEUP22_IRQn                 = 22,
  WAKEUP23_IRQn                 = 23,
  WAKEUP24_IRQn                 = 24,
  WAKEUP25_IRQn                 = 25,
  WAKEUP26_IRQn                 = 26,
  WAKEUP27_IRQn                 = 27,
  WAKEUP28_IRQn                 = 28,
  WAKEUP29_IRQn                 = 29,
  WAKEUP30_IRQn                 = 30,
  WAKEUP31_IRQn                 = 31,
  WAKEUP32_IRQn                 = 32,
  WAKEUP33_IRQn                 = 33,
  WAKEUP34_IRQn                 = 34,
  WAKEUP35_IRQn                 = 35,
  WAKEUP36_IRQn                 = 36,
  WAKEUP37_IRQn                 = 37,
  WAKEUP38_IRQn                 = 38,
  WAKEUP39_IRQn                 = 39,
  I2C_IRQn                      = 40,       /*!< I2C Interrupt                                    */
  TIMER_16_0_IRQn               = 41,       /*!< 16-bit Timer0 Interrupt                          */
  TIMER_16_1_IRQn               = 42,       /*!< 16-bit Timer1 Interrupt                          */
  TIMER_32_0_IRQn               = 43,       /*!< 32-bit Timer0 Interrupt                          */
  TIMER_32_1_IRQn               = 44,       /*!< 32-bit Timer1 Interrupt                          */
  SSP0_IRQn                     = 45,       /*!< SSP Interrupt                                    */
  UART_IRQn                     = 46,       /*!< UART Interrupt                                   */
  USB_IRQn                      = 47,       /*!< USB Regular Interrupt                            */
  USB_FIQn                      = 48,       /*!< USB Fast Interrupt                               */
  ADC_IRQn                      = 49,       /*!< A/D Converter Interrupt                          */
  WDT_IRQn                      = 50,       /*!< Watchdog timer Interrupt                         */
  BOD_IRQn                      = 51,       /*!< Brown Out Detect(BOD) Interrupt                  */
  EINT3_IRQn                    = 53,       /*!< External Interrupt 3 Interrupt                   */
  EINT2_IRQn                    = 54,       /*!< External Interrupt 2 Interrupt                   */
  EINT1_IRQn                    = 55,       /*!< External Interrupt 1 Interrupt                   */
  EINT0_IRQn                    = 56,       /*!< External Interrupt 0 Interrupt                   */
  SSP1_IRQn                     = 57,       /*!< SSP1 Interrupt                                   */
} IRQn_Type;

/*
 * ==========================================================================
 * ----------- Processor and Core Peripheral Section ------------------------
 * ==========================================================================
 */

/* Configuration of the Cortex-M3 Processor and Core Peripherals */
#define __MPU_PRESENT             1         /*!< MPU present or not                               */
#define __NVIC_PRIO_BITS          3         /*!< Number of Bits used for Priority Levels          */
#define __Vendor_SysTickConfig    0         /*!< Set to 1 if different SysTick Config is used     */

/*@}*/ /* end of group LPC13xx_CMSIS */



/******************************************************************************/
/*                Device Specific Peripheral Registers structures             */
/******************************************************************************/

#if defined ( __CC_ARM   )
#pragma anon_unions
#endif

/*------------- System Control (SYSCON) --------------------------------------*/

typedef struct
{ volatile dword SYSMEMREMAP;            /*!< Offset: 0x000 (R/W)  System memory remap Register */
  volatile dword PRESETCTRL;             /*!< Offset: 0x004 (R/W)  Peripheral reset control Register */
  volatile dword SYSPLLCTRL;             /*!< Offset: 0x008 (R/W)  System PLL control Register */
  volatile dword SYSPLLSTAT;             /*!< Offset: 0x00C (R/ )  System PLL status Register */
  volatile dword USBPLLCTRL;             /* USB PLL control, offset 0x10 */
  volatile dword USBPLLSTAT;
           dword RESERVED0[2];

  volatile dword SYSOSCCTRL;             /*!< Offset: 0x020 (R/W)  System oscillator control Register */
  volatile dword WDTOSCCTRL;             /*!< Offset: 0x024 (R/W)  Watchdog oscillator control Register */
  volatile dword IRCCTRL;                /*!< Offset: 0x028 (R/W)  IRC control Register */
           dword RESERVED1[1];
  volatile dword SYSRSTSTAT;             /*!< Offset: 0x030 (R/W)  System reset status Register */
           dword RESERVED2[3];
  volatile dword SYSPLLCLKSEL;           /*!< Offset: 0x040 (R/W)  System PLL clock source select Register */
  volatile dword SYSPLLCLKUEN;           /*!< Offset: 0x044 (R/W)  System PLL clock source update enable Register */
  volatile dword USBPLLCLKSEL;
  volatile dword USBPLLCLKUEN;
           dword RESERVED3[8];

  volatile dword MAINCLKSEL;             /*!< Offset: 0x070 (R/W)  Main clock source select Register */
  volatile dword MAINCLKUEN;             /*!< Offset: 0x074 (R/W)  Main clock source update enable Register */
  volatile dword SYSAHBCLKDIV;           /*!< Offset: 0x078 (R/W)  System AHB clock divider Register */
           dword RESERVED4[1];

  volatile dword SYSAHBCLKCTRL;          /*!< Offset: 0x080 (R/W)  System AHB clock control Register */
           dword RESERVED5[4];
  volatile dword SSP0CLKDIV;             /*!< Offset: 0x094 (R/W)  SSP1 clock divider Register */
  volatile dword UARTCLKDIV;
  volatile dword SSP1CLKDIV;             /*!< Offset: 0x09C (R/W) SSP1 clock divider Register */
           dword RESERVED6[3];

  volatile dword TRACECLKDIV;

  volatile dword SYSTICKCLKDIV;          /*!< Offset: 0x0B0 (R/W) SYSTICK clock divider Register */
           dword RESERVED7[3];

  volatile dword USBCLKSEL;              /*!< Offset: 0x0C0 (R/W)  USB clock source select Register */
  volatile dword USBCLKUEN;              /*!< Offset: 0x0C4 (R/W)  USB clock source update enable Register */
  volatile dword USBCLKDIV;              /*!< Offset: 0x0C8 (R/W)  USB clock source divider Register */
           dword RESERVED8[1];
  volatile dword WDTCLKSEL;              /*!< Offset: 0x0D0 (R/W)  WDT clock source select Register */
  volatile dword WDTCLKUEN;              /*!< Offset: 0x0D4 (R/W)  WDT clock source update enable Register */
  volatile dword WDTCLKDIV;              /*!< Offset: 0x0D8 (R/W)  WDT clock divider Register */
           dword RESERVED9[1];

  volatile dword CLKOUTCLKSEL;           /*!< Offset: 0x0E0 (R/W)  CLKOUT clock source select Register */
  volatile dword CLKOUTUEN;              /*!< Offset: 0x0E4 (R/W)  CLKOUT clock source update enable Register */
  volatile dword CLKOUTDIV;              /*!< Offset: 0x0E8 (R/W)  CLKOUT clock divider Register */
           dword RESERVED10[5];

  volatile dword PIOPORCAP0;             /*!< Offset: 0x100 (R/ )  POR captured PIO status 0 Register */
  volatile dword PIOPORCAP1;             /*!< Offset: 0x104 (R/ )  POR captured PIO status 1 Register */
           dword RESERVED11[11];
           dword RESERVED12[7];
  volatile dword BODCTRL;                /*!< Offset: 0x150 (R/W)  BOD control Register */
           dword RESERVED13[1];
  volatile dword SYSTCKCAL;              /*!< Offset: 0x158 (R/W)  System tick counter calibration Register */
           dword RESERVED14[41];

  volatile dword STARTAPRP0;             /*!< Offset: 0x200 (R/W)  Start logic edge control Register 0 */
  volatile dword STARTERP0;              /*!< Offset: 0x204 (R/W)  Start logic signal enable Register 0 */
  volatile dword STARTRSRP0CLR;          /*!< Offset: 0x208 ( /W)  Start logic reset Register 0 */
  volatile dword STARTSRP0;              /*!< Offset: 0x20C (R/ )  Start logic status Register 0 */
  volatile dword STARTAPRP1;             /*!< Offset: 0x210 (R/W)  Start logic edge control Register 1   (LPC11UXX only) */
  volatile dword STARTERP1;              /*!< Offset: 0x214 (R/W)  Start logic signal enable Register 1  (LPC11UXX only) */
  volatile dword STARTRSRP1CLR;          /*!< Offset: 0x218 ( /W)  Start logic reset Register 1          (LPC11UXX only) */
  volatile dword STARTSRP1;              /*!< Offset: 0x21C (R/ )  Start logic status Register 1         (LPC11UXX only) */
           dword RESERVED17[4];

  volatile dword PDSLEEPCFG;             /*!< Offset: 0x230 (R/W)  Power-down states in Deep-sleep mode Register  */
  volatile dword PDAWAKECFG;             /*!< Offset: 0x234 (R/W)  Power-down states after wake-up from Deep-sleep mode Register*/
  volatile dword PDRUNCFG;               /*!< Offset: 0x238 (R/W)  Power-down configuration Register*/
           dword RESERVED18[110];
  volatile dword DEVICE_ID;              /*!< Offset: 0x3F4 (R/ )  Device ID Register */
} LPC_SYSCON_TypeDef;


/*------------- Pin Connect Block (IOCON) --------------------------------*/
typedef struct
{ volatile dword PIO2_6;                 /*!< Offset: 0x000 (R/W) I/O configuration for pin PIO2_6 */
           dword RESERVED0[1];
  volatile dword PIO2_0;                 /*!< Offset: 0x008 (R/W)  I/O configuration for pin PIO2_0/DTR/SSEL1 */
  volatile dword RESET_PIO0_0;           /*!< Offset: 0x00C (R/W)  I/O configuration for pin RESET/PIO0_0  */
  volatile dword PIO0_1;                 /*!< Offset: 0x010 (R/W)  I/O configuration for pin PIO0_1/CLKOUT/CT32B0_MAT2 */
  volatile dword PIO1_8;                 /*!< Offset: 0x014 (R/W)  I/O configuration for pin PIO1_8/CT16B1_CAP0 */
           dword RESERVED1[1];
  volatile dword PIO0_2;                 /*!< Offset: 0x01C (R/W)  I/O configuration for pin PIO0_2/SSEL0/CT16B0_CAP0 */

  volatile dword PIO2_7;                 /*!< Offset: 0x020 (R/W)  I/O configuration for pin PIO2_7 */
  volatile dword PIO2_8;                 /*!< Offset: 0x024 (R/W)  I/O configuration for pin PIO2_8 */
  volatile dword PIO2_1;                 /*!< Offset: 0x028 (R/W)  I/O configuration for pin PIO2_1/nDSR/SCK1 */
  volatile dword PIO0_3;                 /*!< Offset: 0x02C (R/W)  I/O configuration for pin PIO0_3 */
  volatile dword PIO0_4;                 /*!< Offset: 0x030 (R/W)  I/O configuration for pin PIO0_4/SCL */
  volatile dword PIO0_5;                 /*!< Offset: 0x034 (R/W)  I/O configuration for pin PIO0_5/SDA */
  volatile dword PIO1_9;                 /*!< Offset: 0x038 (R/W)  I/O configuration for pin PIO1_9/CT16B1_MAT0 */
  volatile dword PIO3_4;                 /*!< Offset: 0x03C (R/W)  I/O configuration for pin PIO3_4 */

  volatile dword PIO2_4;                 /*!< Offset: 0x040 (R/W)  I/O configuration for pin PIO2_4 */
  volatile dword PIO2_5;                 /*!< Offset: 0x044 (R/W)  I/O configuration for pin PIO2_5 */
  volatile dword PIO3_5;                 /*!< Offset: 0x048 (R/W)  I/O configuration for pin PIO3_5 */
  volatile dword PIO0_6;                 /*!< Offset: 0x04C (R/W)  I/O configuration for pin PIO0_6/SCK0 */
  volatile dword PIO0_7;                 /*!< Offset: 0x050 (R/W)  I/O configuration for pin PIO0_7/nCTS */
  volatile dword PIO2_9;                 /*!< Offset: 0x054 (R/W)  I/O configuration for pin PIO2_9 */
  volatile dword PIO2_10;                /*!< Offset: 0x058 (R/W)  I/O configuration for pin PIO2_10 */
  volatile dword PIO2_2;                 /*!< Offset: 0x05C (R/W)  I/O configuration for pin PIO2_2/DCD/MISO1 */

  volatile dword PIO0_8;                 /*!< Offset: 0x060 (R/W)  I/O configuration for pin PIO0_8/MISO0/CT16B0_MAT0 */
  volatile dword PIO0_9;                 /*!< Offset: 0x064 (R/W)  I/O configuration for pin PIO0_9/MOSI0/CT16B0_MAT1 */
  volatile dword SWCLK_PIO0_10;          /*!< Offset: 0x068 (R/W)  I/O configuration for pin SWCLK/PIO0_10/SCK0/CT16B0_MAT2 */
  volatile dword PIO1_10;                /*!< Offset: 0x06C (R/W)  I/O configuration for pin PIO1_10/AD6/CT16B1_MAT1 */
  volatile dword PIO2_11;                /*!< Offset: 0x070 (R/W)  I/O configuration for pin PIO2_11/SCK0 */
  volatile dword R_PIO0_11;              /*!< Offset: 0x074 (R/W)  I/O configuration for pin TDI/PIO0_11/AD0/CT32B0_MAT3 */
  volatile dword R_PIO1_0;               /*!< Offset: 0x078 (R/W)  I/O configuration for pin TMS/PIO1_0/AD1/CT32B1_CAP0 */
  volatile dword R_PIO1_1;               /*!< Offset: 0x07C (R/W)  I/O configuration for pin TDO/PIO1_1/AD2/CT32B1_MAT0 */

  volatile dword R_PIO1_2;               /*!< Offset: 0x080 (R/W)  I/O configuration for pin nTRST/PIO1_2/AD3/CT32B1_MAT1 */
  volatile dword PIO3_0;                 /*!< Offset: 0x084 (R/W)  I/O configuration for pin PIO3_0/nDTR */
  volatile dword PIO3_1;                 /*!< Offset: 0x088 (R/W)  I/O configuration for pin PIO3_1/nDSR */
  volatile dword PIO2_3;                 /*!< Offset: 0x08C (R/W)  I/O configuration for pin PIO2_3/RI/MOSI1 */
  volatile dword SWDIO_PIO1_3;           /*!< Offset: 0x090 (R/W)  I/O configuration for pin SWDIO/PIO1_3/AD4/CT32B1_MAT2 */
  volatile dword PIO1_4;                 /*!< Offset: 0x094 (R/W)  I/O configuration for pin PIO1_4/AD5/CT32B1_MAT3 */
  volatile dword PIO1_11;                /*!< Offset: 0x098 (R/W)  I/O configuration for pin PIO1_11/AD7 */
  volatile dword PIO3_2;                 /*!< Offset: 0x09C (R/W)  I/O configuration for pin PIO3_2/nDCD */

  volatile dword PIO1_5;                 /*!< Offset: 0x0A0 (R/W)  I/O configuration for pin PIO1_5/nRTS/CT32B0_CAP0 */
  volatile dword PIO1_6;                 /*!< Offset: 0x0A4 (R/W)  I/O configuration for pin PIO1_6/RXD/CT32B0_MAT0 */
  volatile dword PIO1_7;                 /*!< Offset: 0x0A8 (R/W)  I/O configuration for pin PIO1_7/TXD/CT32B0_MAT1 */
  volatile dword PIO3_3;                 /*!< Offset: 0x0AC (R/W)  I/O configuration for pin PIO3_3/nRI */
  volatile dword SCK_LOC;                /*!< Offset: 0x0B0 (R/W)  SCK pin location select Register */
  volatile dword DSR_LOC;                /*!< Offset: 0x0B4 (R/W)  DSR pin location select Register */
  volatile dword DCD_LOC;                /*!< Offset: 0x0B8 (R/W)  DCD pin location select Register */
  volatile dword RI_LOC;                 /*!< Offset: 0x0BC (R/W)  RI  pin location Register */
} LPC_IOCON_TypeDef;



/*------------- Power Management Unit (PMU) --------------------------*/
typedef struct
{ volatile dword PCON;                   /*!< Offset: 0x000 (R/W)  Power control Register */
  volatile dword GPREG0;                 /*!< Offset: 0x004 (R/W)  General purpose Register 0 */
  volatile dword GPREG1;                 /*!< Offset: 0x008 (R/W)  General purpose Register 1 */
  volatile dword GPREG2;                 /*!< Offset: 0x00C (R/W)  General purpose Register 2 */
  volatile dword GPREG3;                 /*!< Offset: 0x010 (R/W)  General purpose Register 3 */
  volatile dword GPREG4;                 /*!< Offset: 0x014 (R/W)  General purpose Register 4 */
} LPC_PMU_TypeDef;


/*------------- General Purpose Input/Output (GPIO) --------------------------*/
typedef struct
{ union
  { volatile dword MASKED_ACCESS[4096];  /*!< Offset: 0x0000 (R/W) Port data Register for pins PIOn_0 to PIOn_11 */
    struct
    { dword RESERVED0[4095];
      volatile dword DATA;               /*!< Offset: 0x3FFC (R/W) Port data Register */
    };
  };
           dword RESERVED1[4096];
  volatile dword DIR;                    /*!< Offset: 0x8000 (R/W)  Data direction Register */
  volatile dword IS;                     /*!< Offset: 0x8004 (R/W)  Interrupt sense Register */
  volatile dword IBE;                    /*!< Offset: 0x8008 (R/W)  Interrupt both edges Register */
  volatile dword IEV;                    /*!< Offset: 0x800C (R/W)  Interrupt event Register */
  volatile dword IE;                     /*!< Offset: 0x8010 (R/W)  Interrupt mask Register */
  volatile dword RIS;                    /*!< Offset: 0x8014 (R/ )  Raw interrupt status Register */
  volatile dword MIS;                    /*!< Offset: 0x8018 (R/ )  Masked interrupt status Register */
  volatile dword IC;                     /*!< Offset: 0x801C ( /W)  Interrupt clear Register */
} LPC_GPIO_TypeDef;


/*------------- Timer (TMR) --------------------------------------------------*/
typedef struct
{ volatile dword IR;                     /*!< Offset: 0x000 (R/W)  Interrupt Register */
  volatile dword TCR;                    /*!< Offset: 0x004 (R/W)  Timer Control Register */
  volatile dword TC;                     /*!< Offset: 0x008 (R/W)  Timer Counter Register */
  volatile dword PR;                     /*!< Offset: 0x00C (R/W)  Prescale Register */
  volatile dword PC;                     /*!< Offset: 0x010 (R/W)  Prescale Counter Register */
  volatile dword MCR;                    /*!< Offset: 0x014 (R/W)  Match Control Register */
  volatile dword MR0;                    /*!< Offset: 0x018 (R/W)  Match Register 0 */
  volatile dword MR1;                    /*!< Offset: 0x01C (R/W)  Match Register 1 */
  volatile dword MR2;                    /*!< Offset: 0x020 (R/W)  Match Register 2 */
  volatile dword MR3;                    /*!< Offset: 0x024 (R/W)  Match Register 3 */
  volatile dword CCR;                    /*!< Offset: 0x028 (R/W)  Capture Control Register */
  volatile dword CR0;                    /*!< Offset: 0x02C (R/ )  Capture Register 0 */
           dword RESERVED1[3];
  volatile dword EMR;                    /*!< Offset: 0x03C (R/W)  External Match Register */
           dword RESERVED2[12];
  volatile dword CTCR;                   /*!< Offset: 0x070 (R/W)  Count Control Register */
  volatile dword PWMC;                   /*!< Offset: 0x074 (R/W)  PWM Control Register */
} LPC_TMR_TypeDef;


/*------------- Universal Asynchronous Receiver Transmitter (UART) -----------*/
typedef struct
{ union
  { volatile dword  RBR;                 /*!< Offset: 0x000 (R/ )  Receiver Buffer  Register */
    volatile dword  THR;                 /*!< Offset: 0x000 ( /W)  Transmit Holding Register */
    volatile dword  DLL;                 /*!< Offset: 0x000 (R/W)  Divisor Latch LSB */
  };
  union
  { volatile dword  DLM;                 /*!< Offset: 0x004 (R/W)  Divisor Latch MSB */
    volatile dword  IER;                 /*!< Offset: 0x000 (R/W)  Interrupt Enable Register */
  };
  union
  { volatile dword  IIR;                 /*!< Offset: 0x008 (R/ )  Interrupt ID Register */
    volatile dword  FCR;                 /*!< Offset: 0x008 ( /W)  FIFO Control Register */
  };
  volatile dword  LCR;                   /*!< Offset: 0x00C (R/W)  Line Control Register */
  volatile dword  MCR;                   /*!< Offset: 0x010 (R/W)  Modem control Register */
  volatile dword  LSR;                   /*!< Offset: 0x014 (R/ )  Line Status Register */
  volatile dword  MSR;                   /*!< Offset: 0x018 (R/ )  Modem status Register */
  volatile dword  SCR;                   /*!< Offset: 0x01C (R/W)  Scratch Pad Register */
  volatile dword  ACR;                   /*!< Offset: 0x020 (R/W)  Auto-baud Control Register */
           dword  RESERVED0[1];
  volatile dword  FDR;                   /*!< Offset: 0x028 (R/W)  Fractional Divider Register */
           dword  RESERVED1[1];
  volatile dword  TER;                   /*!< Offset: 0x030 (R/W)  Transmit Enable Register */
           dword  RESERVED2[6];
  volatile dword  RS485CTRL;             /*!< Offset: 0x04C (R/W)  RS-485/EIA-485 Control Register */
  volatile dword  ADRMATCH;              /*!< Offset: 0x050 (R/W)  RS-485/EIA-485 address match Register */
  volatile dword  RS485DLY;              /*!< Offset: 0x054 (R/W)  RS-485/EIA-485 direction control delay Register */

} LPC_UART_TypeDef;


/*------------- Synchronous Serial Communication (SSP) -----------------------*/
typedef struct
{ volatile dword CR0;                    /*!< Offset: 0x000 (R/W)  Control Register 0 */
  volatile dword CR1;                    /*!< Offset: 0x004 (R/W)  Control Register 1 */
  volatile dword DR;                     /*!< Offset: 0x008 (R/W)  Data Register */
  volatile dword SR;                     /*!< Offset: 0x00C (R/ )  Status Register */
  volatile dword CPSR;                   /*!< Offset: 0x010 (R/W)  Clock Prescale Register */
  volatile dword IMSC;                   /*!< Offset: 0x014 (R/W)  Interrupt Mask Set and Clear Register */
  volatile dword RIS;                    /*!< Offset: 0x018 (R/ )  Raw Interrupt Status Register */
  volatile dword MIS;                    /*!< Offset: 0x01C (R/ )  Masked Interrupt Status Register */
  volatile dword ICR;                    /*!< Offset: 0x020 ( /W)  SSPICR Interrupt Clear Register */
} LPC_SSP_TypeDef;


/*------------- Inter-Integrated Circuit (I2C) -------------------------------*/
typedef struct
{ volatile dword CONSET;                 /*!< Offset: 0x000 (R/W)  I2C Control Set Register */
  volatile dword STAT;                   /*!< Offset: 0x004 (R/ )  I2C Status Register */
  volatile dword DAT;                    /*!< Offset: 0x008 (R/W)  I2C Data Register */
  volatile dword ADR0;                   /*!< Offset: 0x00C (R/W)  I2C Slave Address Register 0 */
  volatile dword SCLH;                   /*!< Offset: 0x010 (R/W)  SCH Duty Cycle Register High Half Word */
  volatile dword SCLL;                   /*!< Offset: 0x014 (R/W)  SCL Duty Cycle Register Low Half Word */
  volatile dword CONCLR;                 /*!< Offset: 0x018 ( /W)  I2C Control Clear Register */
  volatile dword MMCTRL;                 /*!< Offset: 0x01C (R/W)  Monitor mode control register */
  volatile dword ADR1;                   /*!< Offset: 0x020 (R/W)  I2C Slave Address Register 1 */
  volatile dword ADR2;                   /*!< Offset: 0x024 (R/W)  I2C Slave Address Register 2 */
  volatile dword ADR3;                   /*!< Offset: 0x028 (R/W)  I2C Slave Address Register 3 */
  volatile dword DATA_BUFFER;            /*!< Offset: 0x02C (R/ )  Data buffer Register */
  volatile dword MASK0;                  /*!< Offset: 0x030 (R/W)  I2C Slave address mask register 0 */
  volatile dword MASK1;                  /*!< Offset: 0x034 (R/W)  I2C Slave address mask register 1 */
  volatile dword MASK2;                  /*!< Offset: 0x038 (R/W)  I2C Slave address mask register 2 */
  volatile dword MASK3;                  /*!< Offset: 0x03C (R/W)  I2C Slave address mask register 3 */
} LPC_I2C_TypeDef;


/*------------- Windowed Watchdog Timer (WWDT) -----------------------------------------*/
typedef struct
{ volatile dword MODE;                   /*!< Offset: 0x000 (R/W)  Watchdog mode Register */
  volatile dword TC;                     /*!< Offset: 0x004 (R/W)  Watchdog timer constant Register */
  volatile dword FEED;                   /*!< Offset: 0x008 ( /W)  Watchdog feed sequence Register */
  volatile dword TV;                     /*!< Offset: 0x00C (R/ )  Watchdog timer value Register */
           dword RESERVED0;
  volatile dword WARNINT;		 /*!< Offset: 0x014 (R/W)  Watchdog timer warning int. Register */
  volatile dword WINDOW;		 /*!< Offset: 0x018 (R/W)  Watchdog timer window value Register */
} LPC_WWDT_TypeDef;


/*------------- Analog-to-Digital Converter (ADC) ----------------------------*/
typedef struct
{ volatile dword CR;                     /*!< Offset: 0x000 (R/W)  A/D Control Register */
  volatile dword GDR;                    /*!< Offset: 0x004 (R/W)  A/D Global Data Register */
           dword RESERVED0[1];
  volatile dword INTEN;                  /*!< Offset: 0x00C (R/W)  A/D Interrupt Enable Register */
  volatile dword DR[8];                  /*!< Offset: 0x010 (R/W)  A/D Channel 0..7 Data Register */
  volatile dword STAT;                   /*!< Offset: 0x030 (R/ )  A/D Status Register */
} LPC_ADC_TypeDef;


/*------------- Universal Serial Bus (USB) -----------------------------------*/
typedef struct
{ volatile dword DevIntSt;               /*!< Offset: 0x000 (R/ )  USB Device Interrupt Status Register */
  volatile dword DevIntEn;               /*!< Offset: 0x004 (R/W)  USB Device Interrupt Enable Register */
  volatile dword DevIntClr;              /*!< Offset: 0x008 ( /W)  USB Device Interrupt Clear Register */
  volatile dword DevIntSet;              /*!< Offset: 0x00C ( /W)  USB Device Interrupt Set Register */

  volatile dword CmdCode;                /*!< Offset: 0x010 ( /W)  USB Command Code Register */
  volatile dword CmdData;                /*!< Offset: 0x014 (R/ )  USB Command Data Register */

  volatile dword RxData;                 /*!< Offset: 0x018 (R/ )  USB Receive Data Register */
  volatile dword TxData;                 /*!< Offset: 0x01C ( /W)  USB Transmit Data Register */
  volatile dword RxPLen;                 /*!< Offset: 0x020 (R/ )  USB Receive Packet Length Register */
  volatile dword TxPLen;                 /*!< Offset: 0x024 ( /W)  USB Transmit Packet Length Register */
  volatile dword Ctrl;                   /*!< Offset: 0x028 (R/ )  USB Control Register */
  volatile dword DevFIQSel;              /*!< Offset: 0x02C ( /W)  USB Device FIQ select Register */
} LPC_USB_TypeDef;

#if defined ( __CC_ARM   )
#pragma no_anon_unions
#endif

/******************************************************************************/
/*                         Peripheral memory map                              */
/******************************************************************************/
/* Base addresses                                                             */
#define LPC_FLASH_BASE        (0x00000000UL)
#define LPC_RAM_BASE          (0x10000000UL)
#define LPC_APB0_BASE         (0x40000000UL)
#define LPC_AHB_BASE          (0x50000000UL)

/* APB0 peripherals                                                           */
#define LPC_I2C_BASE          (LPC_APB0_BASE + 0x00000)
#define LPC_WWDT_BASE         (LPC_APB0_BASE + 0x04000)
#define LPC_UART_BASE         (LPC_APB0_BASE + 0x08000)
#define LPC_CT16B0_BASE       (LPC_APB0_BASE + 0x0C000)
#define LPC_CT16B1_BASE       (LPC_APB0_BASE + 0x10000)
#define LPC_CT32B0_BASE       (LPC_APB0_BASE + 0x14000)
#define LPC_CT32B1_BASE       (LPC_APB0_BASE + 0x18000)
#define LPC_ADC_BASE          (LPC_APB0_BASE + 0x1C000)
#define LPC_USB_BASE          (LPC_APB0_BASE + 0x20000)
#define LPC_PMU_BASE          (LPC_APB0_BASE + 0x38000)
#define LPC_SSP0_BASE         (LPC_APB0_BASE + 0x40000)
#define LPC_IOCON_BASE        (LPC_APB0_BASE + 0x44000)
#define LPC_SYSCON_BASE       (LPC_APB0_BASE + 0x48000)
#define LPC_SSP1_BASE         (LPC_APB0_BASE + 0x58000)

/* AHB peripherals                                                            */
#define LPC_GPIO_BASE         (LPC_AHB_BASE  + 0x00000)
#define LPC_GPIO0_BASE        (LPC_AHB_BASE  + 0x00000)
#define LPC_GPIO1_BASE        (LPC_AHB_BASE  + 0x10000)
#define LPC_GPIO2_BASE        (LPC_AHB_BASE  + 0x20000)
#define LPC_GPIO3_BASE        (LPC_AHB_BASE  + 0x30000)

/******************************************************************************/
/*                         Peripheral declaration                             */
/******************************************************************************/
#define LPC_I2C               ((LPC_I2C_TypeDef    *) LPC_I2C_BASE   )
#define LPC_WWDT              ((LPC_WWDT_TypeDef   *) LPC_WWDT_BASE  )
#define LPC_UART              ((LPC_UART_TypeDef   *) LPC_UART_BASE  )
#define LPC_TMR16B0           ((LPC_TMR_TypeDef    *) LPC_CT16B0_BASE)
#define LPC_TMR16B1           ((LPC_TMR_TypeDef    *) LPC_CT16B1_BASE)
#define LPC_TMR32B0           ((LPC_TMR_TypeDef    *) LPC_CT32B0_BASE)
#define LPC_TMR32B1           ((LPC_TMR_TypeDef    *) LPC_CT32B1_BASE)
#define LPC_ADC               ((LPC_ADC_TypeDef    *) LPC_ADC_BASE   )
#define LPC_PMU               ((LPC_PMU_TypeDef    *) LPC_PMU_BASE   )
#define LPC_SSP0              ((LPC_SSP_TypeDef    *) LPC_SSP0_BASE  )
#define LPC_SSP1              ((LPC_SSP_TypeDef    *) LPC_SSP1_BASE  )
#define LPC_IOCON             ((LPC_IOCON_TypeDef  *) LPC_IOCON_BASE )
#define LPC_SYSCON            ((LPC_SYSCON_TypeDef *) LPC_SYSCON_BASE)
#define LPC_USB               ((LPC_USB_TypeDef    *) LPC_USB_BASE   )
#define LPC_GPIO0             ((LPC_GPIO_TypeDef   *) LPC_GPIO0_BASE )
#define LPC_GPIO1             ((LPC_GPIO_TypeDef   *) LPC_GPIO1_BASE )
#define LPC_GPIO2             ((LPC_GPIO_TypeDef   *) LPC_GPIO2_BASE )
#define LPC_GPIO3             ((LPC_GPIO_TypeDef   *) LPC_GPIO3_BASE )

#ifdef __cplusplus
}
#endif

#endif  /* LPC1343INCLUDED */
